@@ -19,12 +19,15 @@ package cl_test
 import (
 	"bytes"
 	"os"
+	"sync"
+	"syscall"
 	"testing"
 
 	"github.com/goplus/gop/cl"
 	"github.com/goplus/gop/cmd/gengo"
 	"github.com/goplus/gop/parser"
 	"github.com/goplus/gop/parser/parsertest"
+	"github.com/goplus/gop/scanner"
 	"github.com/goplus/gop/token"
 	"github.com/goplus/gox"
 	"golang.org/x/tools/go/packages"
@@ -36,30 +39,44 @@ var (
 )
 
 func init() {
-	gox.SetDebug(0)
+	gox.SetDebug(gox.DbgFlagAll)
+	cl.SetDebug(cl.DbgFlagAll)
 	gblFset = token.NewFileSet()
 	baseConf = &cl.Config{
 		Fset:          gblFset,
+		ModPath:       "github.com/goplus/gop",
+		ModRootDir:    ".",
 		GenGoPkg:      new(gengo.Runner).GenGoPkg,
 		CacheLoadPkgs: true,
 		NoFileLine:    true,
 	}
 }
 
-func gopClTest(t *testing.T, gopcode, expected string) {
+func gopClTest(t *testing.T, gopcode, expected string, cachefile ...string) {
+	cl.SetDisableRecover(true)
+	defer cl.SetDisableRecover(false)
+
 	fs := parsertest.NewSingleFileFS("/foo", "bar.gop", gopcode)
 	pkgs, err := parser.ParseFSDir(gblFset, fs, "/foo", nil, 0)
 	if err != nil {
+		scanner.PrintError(os.Stderr, err)
 		t.Fatal("ParseFSDir:", err)
 	}
 	conf := *baseConf.Ensure()
+	if cachefile != nil {
+		copy := *baseConf
+		copy.PkgsLoader = nil
+		copy.CacheFile = cachefile[0]
+		copy.PersistLoadPkgs = true
+		conf = *copy.Ensure()
+	}
 	bar := pkgs["main"]
 	pkg, err := cl.NewPackage("", bar, &conf)
 	if err != nil {
 		t.Fatal("NewPackage:", err)
 	}
 	var b bytes.Buffer
-	err = gox.WriteTo(&b, pkg)
+	err = gox.WriteTo(&b, pkg, false)
 	if err != nil {
 		t.Fatal("gox.WriteTo failed:", err)
 	}
@@ -67,6 +84,24 @@ func gopClTest(t *testing.T, gopcode, expected string) {
 	if result != expected {
 		t.Fatalf("\nResult:\n%s\nExpected:\n%s\n", result, expected)
 	}
+	if cachefile != nil {
+		if err = conf.PkgsLoader.Save(); err != nil {
+			t.Fatal("PkgsLoader.Save failed:", err)
+		}
+	}
+}
+
+func TestEmptyPkgsLoader(t *testing.T) {
+	l := &cl.PkgsLoader{}
+	if l.Save() != nil {
+		t.Fatal("PkgsLoader.Save failed")
+	}
+	if l.GenGoPkgs(nil, nil) != syscall.ENOENT {
+		t.Fatal("PkgsLoader.GenGoPkgs failed")
+	}
+	if _, err := cl.GetModulePath("/dir-not-exists/go.mod"); err == nil {
+		t.Fatal("GetModulePath failed")
+	}
 }
 
 func TestNilConf(t *testing.T) {
@@ -223,7 +258,6 @@ func foo(shape Shape) {
 `)
 }
 
-// TODO: why Area is comming before Circle?
 func TestInterfaceExample(t *testing.T) {
 	gopClTest(t, `
 type Shape interface {
@@ -311,7 +345,7 @@ type BigInt struct {
 import big "math/big"
 
 type BigInt struct {
-	Int *big.Int
+	*big.Int
 }
 `)
 }
@@ -334,65 +368,20 @@ import (
 )
 
 func foo(script string) {
-	var _autoGop_1 gopq.NodeSet
-	{
+	doc := func() (_gop_ret gopq.NodeSet) {
 		var _gop_err error
-		_autoGop_1, _gop_err = goptest.New(script)
+		_gop_ret, _gop_err = goptest.New(script)
 		if _gop_err != nil {
 			panic(_gop_err)
 		}
-		goto _autoGop_2
-	_autoGop_2:
-	}
-	doc := _autoGop_1
+		return
+	}()
 	fmt.Println(doc.Any().FuncDecl().Name())
 	fmt.Println(doc.Any().ImportSpec().Name())
 }
 `)
 }
 
-func TestImportGopPkg(t *testing.T) {
-	os.Remove("../tutorial/14-Using-goplus-in-Go/foo/gop_autogen.go")
-	gopClTest(t, `import "github.com/goplus/gop/tutorial/14-Using-goplus-in-Go/foo"
-
-rmap := foo.ReverseMap(map[string]int{"Hi": 1, "Hello": 2})
-println(rmap)
-`, `package main
-
-import (
-	fmt "fmt"
-	foo "github.com/goplus/gop/tutorial/14-Using-goplus-in-Go/foo"
-)
-
-func main() {
-	rmap := foo.ReverseMap(map[string]int{"Hi": 1, "Hello": 2})
-	fmt.Println(rmap)
-}
-`)
-}
-
-// bugfix (only depends order of testing functions)
-// vet: open tutorial/14-Using-goplus-in-Go/foo/gop_autogen.go: no such file or directory
-func TestGopkgDep(t *testing.T) {
-	os.Remove("../tutorial/14-Using-goplus-in-Go/foo/gop_autogen.go")
-	const (
-		loadTypes = packages.NeedImports | packages.NeedDeps | packages.NeedTypes
-		loadModes = loadTypes | packages.NeedName | packages.NeedModule
-	)
-	loadConf := &packages.Config{Mode: loadModes, Fset: gblFset}
-	pkgs, err := baseConf.Ensure().PkgsLoader.Load(
-		loadConf, "github.com/goplus/gop/tutorial/14-Using-goplus-in-Go/gomain")
-	if err != nil {
-		t.Fatal("PkgsLoader.Load failed:", err)
-	}
-	for _, err = range pkgs[0].Errors {
-		t.Fatal("PkgsLoader.Load failed:", err)
-	}
-	if pkgs[0].Name != "main" {
-		t.Fatal("pkg name:", pkgs[0].Name)
-	}
-}
-
 func TestErrWrap(t *testing.T) {
 	gopClTest(t, `
 import "strconv"
@@ -442,29 +431,21 @@ func addSafe(x, y string) int {
 import strconv "strconv"
 
 func addSafe(x string, y string) int {
-	var _autoGop_1 int
-	{
+	return func() (_gop_ret int) {
 		var _gop_err error
-		_autoGop_1, _gop_err = strconv.Atoi(x)
+		_gop_ret, _gop_err = strconv.Atoi(x)
 		if _gop_err != nil {
-			_autoGop_1 = 0
-			goto _autoGop_2
+			return 0
 		}
-		goto _autoGop_2
-	_autoGop_2:
-	}
-	var _autoGop_3 int
-	{
+		return
+	}() + func() (_gop_ret int) {
 		var _gop_err error
-		_autoGop_3, _gop_err = strconv.Atoi(y)
+		_gop_ret, _gop_err = strconv.Atoi(y)
 		if _gop_err != nil {
-			_autoGop_3 = 0
-			goto _autoGop_4
+			return 0
 		}
-		goto _autoGop_4
-	_autoGop_4:
-	}
-	return _autoGop_1 + _autoGop_3
+		return
+	}()
 }
 `)
 }
@@ -726,6 +707,101 @@ func main() {
 `)
 }
 
+func TestCompositeLit2(t *testing.T) {
+	gopClTest(t, `
+type foo struct {
+	A int
+}
+
+x := []*struct{a int}{
+	{1}, {3}, {5},
+}
+y := map[foo]struct{a string}{
+	{1}: {"Hi"},
+}
+z := [...]foo{
+	{1}, {3}, {5},
+}
+`, `package main
+
+type foo struct {
+	A int
+}
+
+func main() {
+	x := []*struct {
+		a int
+	}{&struct {
+		a int
+	}{1}, &struct {
+		a int
+	}{3}, &struct {
+		a int
+	}{5}}
+	y := map[foo]struct {
+		a string
+	}{foo{1}: struct {
+		a string
+	}{"Hi"}}
+	z := [...]foo{foo{1}, foo{3}, foo{5}}
+}
+`)
+}
+
+// deduce struct type as parameters of a function call
+func TestCompositeLit3(t *testing.T) {
+	gopClTest(t, `
+type Config struct {
+	A int
+}
+
+func foo(conf *Config) {
+}
+
+func bar(conf ...Config) {
+}
+
+foo({A: 1})
+bar({A: 2})
+`, `package main
+
+type Config struct {
+	A int
+}
+
+func foo(conf *Config) {
+}
+func bar(conf ...Config) {
+}
+func main() {
+	foo(&Config{A: 1})
+	bar(Config{A: 2})
+}
+`)
+}
+
+// deduce struct type as results of a function call
+func TestCompositeLit4(t *testing.T) {
+	gopClTest(t, `
+type Result struct {
+	A int
+}
+
+func foo() *Result {
+	return {A: 1}
+}
+`, `package main
+
+type Result struct {
+	A int
+}
+
+func foo() *Result {
+	return &Result{A: 1}
+}
+`)
+}
+
 func TestSliceLit(t *testing.T) {
 	gopClTest(t, `
 x := [1, 3.4, 5]
@@ -926,6 +1002,208 @@ func main() {
 `)
 }
 
+func TestRangeStmtUDT(t *testing.T) {
+	gopClTest(t, `
+type foo struct {
+}
+
+func (p *foo) Gop_Enum(c func(key int, val string)) {
+}
+
+for k, v := range new(foo) {
+	println(k, v)
+}
+`, `package main
+
+import fmt "fmt"
+
+type foo struct {
+}
+
+func (p *foo) Gop_Enum(c func(key int, val string)) {
+}
+func main() {
+	new(foo).Gop_Enum(func(k int, v string) {
+		fmt.Println(k, v)
+	})
+}
+`)
+}
+
+func TestForPhraseUDT(t *testing.T) {
+	gopClTest(t, `
+type foo struct {
+}
+
+func (p *foo) Gop_Enum(c func(val string)) {
+}
+
+for v <- new(foo) {
+	println(v)
+}
+`, `package main
+
+import fmt "fmt"
+
+type foo struct {
+}
+
+func (p *foo) Gop_Enum(c func(val string)) {
+}
+func main() {
+	new(foo).Gop_Enum(func(v string) {
+		fmt.Println(v)
+	})
+}
+`)
+}
+
+func TestForPhraseUDT2(t *testing.T) {
+	gopClTest(t, `
+type fooIter struct {
+}
+
+func (p fooIter) Next() (key string, val int, ok bool) {
+	return
+}
+
+type foo struct {
+}
+
+func (p *foo) Gop_Enum() fooIter {
+}
+
+for k, v <- new(foo) {
+	println(k, v)
+}
+`, `package main
+
+import fmt "fmt"
+
+type fooIter struct {
+}
+
+func (p fooIter) Next() (key string, val int, ok bool) {
+	return
+}
+
+type foo struct {
+}
+
+func (p *foo) Gop_Enum() fooIter {
+}
+func main() {
+	for _gop_it := new(foo).Gop_Enum(); ; {
+		var _gop_ok bool
+		k, v, _gop_ok := _gop_it.Next()
+		if !_gop_ok {
+			break
+		}
+		fmt.Println(k, v)
+	}
+}
+`)
+}
+
+func TestForPhraseUDT3(t *testing.T) {
+	gopClTest(t, `
+type foo struct {
+}
+
+func (p *foo) Gop_Enum(c func(val string)) {
+}
+
+println([v for v <- new(foo)])
+`, `package main
+
+import fmt "fmt"
+
+type foo struct {
+}
+
+func (p *foo) Gop_Enum(c func(val string)) {
+}
+func main() {
+	fmt.Println(func() (_gop_ret []string) {
+		new(foo).Gop_Enum(func(v string) {
+			_gop_ret = append(_gop_ret, v)
+		})
+		return
+	}())
+}
+`)
+}
+
+func TestForPhraseUDT4(t *testing.T) {
+	gopClTest(t, `
+type fooIter struct {
+	data *foo
+	idx  int
+}
+
+func (p *fooIter) Next() (key int, val string, ok bool) {
+	if p.idx < len(p.data.key) {
+		key, val, ok = p.data.key[p.idx], p.data.val[p.idx], true
+		p.idx++
+	}
+	return
+}
+
+type foo struct {
+	key []int
+	val []string
+}
+
+func newFoo() *foo {
+	return &foo{key: [3, 7], val: ["Hi", "Go+"]}
+}
+
+func (p *foo) Gop_Enum() *fooIter {
+	return &fooIter{data: p}
+}
+
+for k, v <- newFoo() {
+	println(k, v)
+}
+`, `package main
+
+import fmt "fmt"
+
+type fooIter struct {
+	data *foo
+	idx  int
+}
+type foo struct {
+	key []int
+	val []string
+}
+
+func (p *fooIter) Next() (key int, val string, ok bool) {
+	if p.idx < len(p.data.key) {
+		key, val, ok = p.data.key[p.idx], p.data.val[p.idx], true
+		p.idx++
+	}
+	return
+}
+func (p *foo) Gop_Enum() *fooIter {
+	return &fooIter{data: p}
+}
+func newFoo() *foo {
+	return &foo{key: []int{3, 7}, val: []string{"Hi", "Go+"}}
+}
+func main() {
+	for _gop_it := newFoo().Gop_Enum(); ; {
+		var _gop_ok bool
+		k, v, _gop_ok := _gop_it.Next()
+		if !_gop_ok {
+			break
+		}
+		fmt.Println(k, v)
+	}
+}
+`)
+}
+
 func TestForPhrase(t *testing.T) {
 	gopClTest(t, `
 sum := 0
@@ -991,6 +1269,25 @@ func main() {
 `)
 }
 
+func TestMapComprehensionCond2(t *testing.T) {
+	gopClTest(t, `
+z := {t: k for k, v <- {"Hello": 1, "Hi": 3, "xsw": 5, "Go+": 7}, t := v; t > 3}
+`, `package main
+
+func main() {
+	z := func() (_gop_ret map[int]string) {
+		_gop_ret = map[int]string{}
+		for k, v := range map[string]int{"Hello": 1, "Hi": 3, "xsw": 5, "Go+": 7} {
+			if t := v; t > 3 {
+				_gop_ret[t] = k
+			}
+		}
+		return
+	}()
+}
+`)
+}
+
 func TestExistsComprehension(t *testing.T) {
 	gopClTest(t, `
 hasFive := {for x <- ["1", "3", "5", "7", "11"], x == "5"}
@@ -1223,6 +1520,10 @@ type M int
 func (m M) Foo() {
 	println("foo", m)
 }
+
+func (M) Bar() {
+	println("bar")
+}
 `, `package main
 
 import fmt "fmt"
@@ -1232,6 +1533,9 @@ type M int
 func (m M) Foo() {
 	fmt.Println("foo", m)
 }
+func ( M) Bar() {
+	fmt.Println("bar")
+}
 `)
 }
 
@@ -1315,6 +1619,19 @@ func main() {
 `)
 }
 
+func TestImportForceUsed(t *testing.T) {
+	gopClTest(t, `import _ "fmt"
+
+func main() {
+}`, `package main
+
+import _ "fmt"
+
+func main() {
+}
+`)
+}
+
 func TestAnonymousImport(t *testing.T) {
 	gopClTest(t, `println("Hello")
 printf("Hello Go+\n")
@@ -1579,6 +1896,35 @@ func main() {
 `)
 }
 
+func TestLambdaExpr(t *testing.T) {
+	gopClTest(t, `
+func Map(c []float64, t func(float64) float64) {
+	// ...
+}
+
+func Map2(c []float64, t func(float64) (float64, float64)) {
+	// ...
+}
+
+Map([1.2, 3.5, 6], x => x * x)
+Map2([1.2, 3.5, 6], x => (x * x, x + x))
+`, `package main
+
+func Map(c []float64, t func(float64) float64) {
+}
+func Map2(c []float64, t func(float64) (float64, float64)) {
+}
+func main() {
+	Map([]float64{1.2, 3.5, 6}, func(x float64) float64 {
+		return x * x
+	})
+	Map2([]float64{1.2, 3.5, 6}, func(x float64) (float64, float64) {
+		return x * x, x + x
+	})
+}
+`)
+}
+
 func TestUnnamedMainFunc(t *testing.T) {
 	gopClTest(t, `i := 1`, `package main
 
@@ -1765,3 +2111,137 @@ func main() {
 }
 `)
 }
+
+var (
+	autogen sync.Mutex
+)
+
+func removeAutogenFiles() {
+	os.Remove("../tutorial/14-Using-goplus-in-Go/foo/gop_autogen.go")
+	os.Remove("../tutorial/14-Using-goplus-in-Go/foo/gop_autogen_test.go")
+	os.Remove("../tutorial/14-Using-goplus-in-Go/foo/gop_autogen2_test.go")
+}
+
+func TestImportGopPkg(t *testing.T) {
+	autogen.Lock()
+	defer autogen.Unlock()
+
+	removeAutogenFiles()
+	gopClTest(t, `import "github.com/goplus/gop/tutorial/14-Using-goplus-in-Go/foo"
+
+rmap := foo.ReverseMap(map[string]int{"Hi": 1, "Hello": 2})
+println(rmap)
+`, `package main
+
+import (
+	fmt "fmt"
+	foo "github.com/goplus/gop/tutorial/14-Using-goplus-in-Go/foo"
+)
+
+func main() {
+	rmap := foo.ReverseMap(map[string]int{"Hi": 1, "Hello": 2})
+	fmt.Println(rmap)
+}
+`)
+}
+
+// bugfix (only depends order of testing functions)
+// vet: open tutorial/14-Using-goplus-in-Go/foo/gop_autogen.go: no such file or directory
+func TestGopkgDep(t *testing.T) {
+	autogen.Lock()
+	defer autogen.Unlock()
+
+	removeAutogenFiles()
+	const (
+		loadTypes = packages.NeedImports | packages.NeedDeps | packages.NeedTypes
+		loadModes = loadTypes | packages.NeedName | packages.NeedModule
+	)
+	loadConf := &packages.Config{Mode: loadModes, Fset: gblFset}
+	pkgs, err := baseConf.Ensure().PkgsLoader.Load(
+		loadConf, "github.com/goplus/gop/tutorial/14-Using-goplus-in-Go/gomain")
+	if err != nil {
+		t.Fatal("PkgsLoader.Load failed:", err)
+	}
+	for _, err = range pkgs[0].Errors {
+		t.Fatal("PkgsLoader.Load failed:", err)
+	}
+	if pkgs[0].Name != "main" {
+		t.Fatal("pkg name:", pkgs[0].Name)
+	}
+}
+
+func TestCallDep(t *testing.T) {
+	const (
+		cachedir  = "../.gop"
+		cachefile = cachedir + "/gop.cache"
+	)
+	os.Remove(cachefile)
+	defer func() {
+		os.Remove(cachefile)
+		os.Remove(cachedir)
+	}()
+	for i := 0; i < 2; i++ {
+		gopClTest(t, `
+import (
+	"reflect"
+	"testing"
+)
+
+func TestNew(t *testing.T) {
+	ret := New()
+	expected := Result{}
+	if reflect.DeepEqual(ret, expected) {
+		t.Fatal("Test failed:", ret, expected)
+	}
+}
+
+type Repo struct {
+	Title string
+}
+
+func newRepo() Repo {
+	return {Title: "Hi"}
+}
+
+type Result struct {
+	Repo Repo
+}
+
+func New() Result {
+	repo := newRepo()
+	return {Repo: repo}
+}
+`, `package main
+
+import (
+	reflect "reflect"
+	testing "testing"
+)
+
+func TestNew(t *testing.T) {
+	ret := New()
+	expected := Result{}
+	if reflect.DeepEqual(ret, expected) {
+		t.Fatal("Test failed:", ret, expected)
+	}
+}
+
+type Result struct {
+	Repo Repo
+}
+
+func New() Result {
+	repo := newRepo()
+	return Result{Repo: repo}
+}
+
+type Repo struct {
+	Title string
+}
+
+func newRepo() Repo {
+	return Repo{Title: "Hi"}
+}
+`, "")
+	}
+}
