@@ -30,7 +30,16 @@ import (
 	"github.com/goplus/gox"
 )
 
-// -----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
+
+Name lookup:
+- local context variables
+- $recv members (only in class files)
+- package globals (variables, constants, types, imported packages etc.)
+- $spx package exports (only in class files)
+- $universe package exports (including builtins)
+
+// ---------------------------------------------------------------------------*/
 
 func compileExprLHS(ctx *blockCtx, expr ast.Expr) {
 	switch v := expr.(type) {
@@ -50,7 +59,7 @@ func compileExprLHS(ctx *blockCtx, expr ast.Expr) {
 func compileExpr(ctx *blockCtx, expr ast.Expr, twoValue ...bool) {
 	switch v := expr.(type) {
 	case *ast.Ident:
-		compileIdent(ctx, v, false)
+		compileIdent(ctx, v, false, true)
 	case *ast.BasicLit:
 		compileBasicLit(ctx, v)
 	case *ast.CallExpr:
@@ -64,7 +73,7 @@ func compileExpr(ctx *blockCtx, expr ast.Expr, twoValue ...bool) {
 	case *ast.FuncLit:
 		compileFuncLit(ctx, v)
 	case *ast.CompositeLit:
-		compileCompositeLit(ctx, v)
+		compileCompositeLit(ctx, v, nil, false)
 	case *ast.SliceLit:
 		compileSliceLit(ctx, v)
 	case *ast.IndexExpr:
@@ -165,13 +174,19 @@ func compileSliceExpr(ctx *blockCtx, v *ast.SliceExpr) { // x[i:j:k]
 func compileSelectorExprLHS(ctx *blockCtx, v *ast.SelectorExpr) {
 	switch x := v.X.(type) {
 	case *ast.Ident:
-		o, at := lookupParent(ctx, x.Name)
+		if compileClassMember(ctx, x, x.Name, false, true) {
+			break
+		}
+		o, at, builtin := lookupParent(ctx, x.Name)
 		if at != nil {
 			ctx.cb.VarRef(at.Ref(v.Sel.Name))
 			return
 		}
+		if isBuiltin(builtin) {
+			panic(ctx.newCodeErrorf(x.Pos(), "use of builtin %s not in function call", x.Name))
+		}
 		if o == nil {
-			log.Panicln("TODO: ident not found -", x.Name)
+			panic(ctx.newCodeErrorf(x.Pos(), "undefined: %s", x.Name))
 		}
 		ctx.cb.Val(o)
 	default:
@@ -180,55 +195,166 @@ func compileSelectorExprLHS(ctx *blockCtx, v *ast.SelectorExpr) {
 	ctx.cb.MemberRef(v.Sel.Name, v)
 }
 
+func isBuiltin(o types.Object) bool {
+	if _, ok := o.(*types.Builtin); ok {
+		return ok
+	}
+	return false
+}
+
 func compileSelectorExpr(ctx *blockCtx, v *ast.SelectorExpr, autoCall bool) {
-	cb := ctx.cb
 	switch x := v.X.(type) {
 	case *ast.Ident:
-		o, at := lookupParent(ctx, x.Name)
+		if compileClassMember(ctx, x, x.Name, autoCall, false) {
+			break
+		}
+		o, at, builtin := lookupParent(ctx, x.Name)
 		if at != nil {
-			cb.Val(at.Ref(v.Sel.Name))
-			return
+			if compilePkgRef(ctx, at, v.Sel.Name, false) {
+				return
+			}
+			if token.IsExported(v.Sel.Name) {
+				panic(ctx.newCodeErrorf(x.Pos(), "undefined: %s.%s", x.Name, v.Sel.Name))
+			} else {
+				panic(ctx.newCodeErrorf(x.Pos(), "cannot refer to unexported name %s.%s", x.Name, v.Sel.Name))
+			}
+		} else if isBuiltin(builtin) {
+			panic(ctx.newCodeErrorf(x.Pos(), "use of builtin %s not in function call", x.Name))
+		} else if o == nil {
+			panic(ctx.newCodeErrorf(x.Pos(), "undefined: %s", x.Name))
 		}
-		if o == nil {
-			log.Panicln("TODO: ident not found -", x.Name)
-		}
-		cb.Val(o)
+		ctx.cb.Val(o)
 	default:
 		compileExpr(ctx, v.X)
 	}
-	name := v.Sel.Name
-	kind, err := cb.Member(name, v)
-	if kind != 0 {
-		return
+	if err := compileMember(ctx, v, v.Sel.Name, autoCall, false); err != nil {
+		panic(err)
+	}
+}
+
+func compilePkgRef(ctx *blockCtx, at *gox.PkgRef, name string, lhs bool) bool {
+	v := pkgRef(at, name)
+	if v == nil {
+		return false
+	}
+	if lhs {
+		ctx.cb.VarRef(v)
+	} else {
+		ctx.cb.Val(v)
+	}
+	return true
+}
+
+func pkgRef(at *gox.PkgRef, name string) gox.Ref {
+	if o := at.Ref(name); o != nil {
+		return o
 	}
 	if c := name[0]; c >= 'a' && c <= 'z' {
 		name = string(rune(c)+('A'-'a')) + name[1:]
-		switch kind, _ = cb.Member(name, v); kind {
+		return at.Ref(name)
+	}
+	return nil
+}
+
+func compileMember(ctx *blockCtx, v ast.Node, name string, autoCall, lhs bool) error {
+	cb := ctx.cb
+	kind, err := cb.Member(name, lhs, v)
+	if kind != 0 {
+		return nil
+	}
+	if c := name[0]; c >= 'a' && c <= 'z' {
+		name = string(rune(c)+('A'-'a')) + name[1:]
+		switch kind, _ = cb.Member(name, lhs, v); kind {
 		case gox.MemberMethod:
 			if autoCall {
 				cb.Call(0)
 			}
-			return
+			return nil
 		case gox.MemberField:
-			return
+			return nil
 		}
 	}
-	panic(err)
+	return err
+}
+
+type fnType struct {
+	params   *types.Tuple
+	n1       int
+	variadic bool
+}
+
+func (p *fnType) init(t *types.Signature) {
+	p.params, p.variadic = t.Params(), t.Variadic()
+	p.n1 = p.params.Len()
+	if p.variadic {
+		p.n1--
+	}
+}
+
+func (p *fnType) arg(i int, ellipsis bool) types.Type {
+	if i < p.n1 {
+		return p.params.At(i).Type()
+	}
+	if p.variadic {
+		t := p.params.At(p.n1).Type()
+		if ellipsis {
+			return t
+		}
+		return t.(*types.Slice).Elem()
+	}
+	return nil
 }
 
 func compileCallExpr(ctx *blockCtx, v *ast.CallExpr) {
 	switch fn := v.Fun.(type) {
 	case *ast.Ident:
-		compileIdent(ctx, fn, true)
+		compileIdent(ctx, fn, true, false)
 	case *ast.SelectorExpr:
 		compileSelectorExpr(ctx, fn, false)
 	default:
 		compileExpr(ctx, fn)
 	}
-	for _, arg := range v.Args {
-		compileExpr(ctx, arg)
+	var fn fnType
+	switch t := ctx.cb.Get(-1).Type.(type) {
+	case *types.Signature:
+		fn.init(t)
 	}
-	ctx.cb.CallWith(len(v.Args), v.Ellipsis != gotoken.NoPos, v)
+	ellipsis := (v.Ellipsis != gotoken.NoPos)
+	for i, arg := range v.Args {
+		if l, ok := arg.(*ast.LambdaExpr); ok {
+			in := fn.arg(i, true).(*types.Signature).Params()
+			compileLambdaExpr(ctx, l, in)
+			continue
+		}
+		if c, ok := arg.(*ast.CompositeLit); ok && c.Type == nil {
+			compileCompositeLit(ctx, c, fn.arg(i, ellipsis), true)
+		} else {
+			compileExpr(ctx, arg)
+		}
+	}
+	ctx.cb.CallWith(len(v.Args), ellipsis, v)
+}
+
+func compileLambdaExpr(ctx *blockCtx, v *ast.LambdaExpr, in *types.Tuple) {
+	n := len(v.Lhs)
+	if n != in.Len() {
+		log.Panicln("TODO: compileLambdaExpr - unexpected argument count", n, in.Len())
+	}
+	pkg := ctx.pkg
+	params := make([]*types.Var, n)
+	for i, name := range v.Lhs {
+		params[i] = pkg.NewParam(token.NoPos, name.Name, in.At(i).Type())
+	}
+	nout := len(v.Rhs)
+	results := make([]*types.Var, nout)
+	for i := 0; i < nout; i++ {
+		results[i] = pkg.NewAutoParam("")
+	}
+	ctx.cb.NewClosure(types.NewTuple(params...), types.NewTuple(results...), false).BodyStart(pkg)
+	for _, v := range v.Rhs {
+		compileExpr(ctx, v)
+	}
+	ctx.cb.Return(nout).End()
 }
 
 func compileFuncLit(ctx *blockCtx, v *ast.FuncLit) {
@@ -247,46 +373,87 @@ func compileIdentLHS(ctx *blockCtx, v *ast.Ident) {
 	if name == "_" {
 		ctx.cb.VarRef(nil)
 	} else {
+		if compileClassMember(ctx, v, name, false, true) {
+			return
+		}
 		_, o := ctx.cb.Scope().LookupParent(name, gotoken.NoPos)
 		if o == nil {
-			panic("TODO: var not found")
+			panic(ctx.newCodeErrorf(v.Pos(), "undefined: %s", name))
 		}
 		ctx.cb.VarRef(o, v)
 	}
 }
 
-func compileIdent(ctx *blockCtx, ident *ast.Ident, allowBuiltin bool) {
+func compileIdent(ctx *blockCtx, ident *ast.Ident, allowBuiltin, autoCall bool) {
 	name := ident.Name
 	if name == "_" {
-		log.Panicln("TODO: cannot use _ as value")
+		panic(ctx.newCodeError(ident.Pos(), "cannot use _ as value"))
 	}
-	if o, _ := lookupParent(ctx, name); o != nil {
-		if !allowBuiltin {
-			if _, ok := o.(*types.Builtin); ok {
-				panic("unexpected builtin: " + name)
-			}
+	if compileClassMember(ctx, ident, name, autoCall, false) {
+		return
+	}
+	if o, _, builtin := lookupParent(ctx, name); o != nil {
+		if !allowBuiltin && isBuiltin(builtin) {
+			panic(ctx.newCodeErrorf(ident.Pos(), "use of builtin %s not in function call", name))
 		}
 		ctx.cb.Val(o, ident)
 	} else {
-		log.Panicln("TODO: var not found -", name)
+		panic(ctx.newCodeErrorf(ident.Pos(), "undefined: %s", name))
 	}
 }
 
-func lookupParent(ctx *blockCtx, name string) (types.Object, *gox.PkgRef) {
+func compileClassMember(ctx *blockCtx, v ast.Node, name string, autoCall, lhs bool) bool {
+	if ctx.fileType > 0 {
+		if fn := ctx.cb.Func(); fn != nil {
+			sig := fn.Type().(*types.Signature)
+			if recv := sig.Recv(); recv != nil {
+				ctx.cb.Val(recv)
+				if compileMember(ctx, v, name, autoCall, lhs) == nil {
+					return true
+				}
+				ctx.cb.InternalStack().PopN(1)
+				if compilePkgRef(ctx, ctx.spx, name, lhs) {
+					return true
+				}
+			}
+		}
+	}
+	return false
+}
+
+// obj, pkgRef, objBuiltin
+func lookupParent(ctx *blockCtx, name string) (types.Object, *gox.PkgRef, types.Object) {
 	at, o := ctx.cb.Scope().LookupParent(name, token.NoPos)
 	if o != nil && at != types.Universe {
-		return o, nil
+		if debugLookup {
+			log.Println("==> LookupParent", name, "=>", o)
+		}
+		return o, nil, nil
 	}
 	if ctx.loadSymbol(name) {
-		return ctx.pkg.Types.Scope().Lookup(name), nil
+		if v := ctx.pkg.Types.Scope().Lookup(name); v != nil {
+			if debugLookup {
+				log.Println("==> Lookup (LoadSymbol)", name, "=>", v)
+			}
+			return v, nil, nil
+		}
 	}
 	if pkgRef, ok := ctx.imports[name]; ok {
-		return nil, pkgRef
+		if debugLookup {
+			log.Println("==> Lookup (ImportPkgs)", name)
+		}
+		return nil, pkgRef, nil
 	}
 	if obj := ctx.pkg.Builtin().Ref(name); obj != nil {
-		return obj, nil
+		if debugLookup {
+			log.Println("==> Lookup (Builtin)", name)
+		}
+		return obj, nil, o
 	}
-	return o, nil
+	if debugLookup && o != nil {
+		log.Println("==> Lookup (Universe)", name)
+	}
+	return o, nil, o
 }
 
 func compileBasicLit(ctx *blockCtx, v *ast.BasicLit) {
@@ -313,16 +480,28 @@ func checkCompositeLitElts(ctx *blockCtx, elts []ast.Expr) (kind int) {
 	return compositeLitVal
 }
 
-func compileCompositeLitElts(ctx *blockCtx, elts []ast.Expr, kind int) {
+func compileCompositeLitElts(ctx *blockCtx, elts []ast.Expr, kind int, expected *kvType) {
 	for _, elt := range elts {
 		if kv, ok := elt.(*ast.KeyValueExpr); ok {
-			compileExpr(ctx, kv.Key)
-			compileExpr(ctx, kv.Value)
+			if key, ok := kv.Key.(*ast.CompositeLit); ok && key.Type == nil {
+				compileCompositeLit(ctx, key, expected.Key(), false)
+			} else {
+				compileExpr(ctx, kv.Key)
+			}
+			if val, ok := kv.Value.(*ast.CompositeLit); ok && val.Type == nil {
+				compileCompositeLit(ctx, val, expected.Elem(), false)
+			} else {
+				compileExpr(ctx, kv.Value)
+			}
 		} else {
 			if kind == compositeLitKeyVal {
 				ctx.cb.None()
 			}
-			compileExpr(ctx, elt)
+			if val, ok := elt.(*ast.CompositeLit); ok && val.Type == nil {
+				compileCompositeLit(ctx, val, expected.Elem(), false)
+			} else {
+				compileExpr(ctx, elt)
+			}
 		}
 	}
 }
@@ -350,20 +529,81 @@ func lookupField(t *types.Struct, name string) int {
 	return -1
 }
 
-func compileCompositeLit(ctx *blockCtx, v *ast.CompositeLit) {
+type kvType struct {
+	underlying types.Type
+	key, val   types.Type
+	cached     bool
+}
+
+func (p *kvType) required() *kvType {
+	if !p.cached {
+		p.cached = true
+		switch t := p.underlying.(type) {
+		case *types.Slice:
+			p.key, p.val = types.Typ[types.Int], t.Elem()
+		case *types.Array:
+			p.key, p.val = types.Typ[types.Int], t.Elem()
+		case *types.Map:
+			p.key, p.val = t.Key(), t.Elem()
+		}
+	}
+	return p
+}
+
+func (p *kvType) Key() types.Type {
+	return p.required().key
+}
+
+func (p *kvType) Elem() types.Type {
+	return p.required().val
+}
+
+func getUnderlying(ctx *blockCtx, typ types.Type) types.Type {
+	u := typ.Underlying()
+	if u == nil {
+		if t, ok := typ.(*types.Named); ok {
+			ctx.loadNamed(ctx.pkg, t)
+			u = t.Underlying()
+		}
+	}
+	return u
+}
+
+func compileCompositeLit(ctx *blockCtx, v *ast.CompositeLit, expected types.Type, onlyStruct bool) {
+	var hasPtr bool
 	var typ, underlying types.Type
+	var kind = checkCompositeLitElts(ctx, v.Elts)
 	if v.Type != nil {
 		typ = toType(ctx, v.Type)
 		underlying = typ.Underlying()
+	} else if expected != nil {
+		if t, ok := expected.(*types.Pointer); ok {
+			expected, hasPtr = t.Elem(), true
+		}
+		if onlyStruct {
+			if kind == compositeLitKeyVal {
+				t := getUnderlying(ctx, expected)
+				if _, ok := t.(*types.Struct); ok { // can't omit non-struct type
+					typ, underlying = expected, t
+				}
+			}
+		} else {
+			typ, underlying = expected, getUnderlying(ctx, expected)
+		}
 	}
-	kind := checkCompositeLitElts(ctx, v.Elts)
 	if t, ok := underlying.(*types.Struct); ok && kind == compositeLitKeyVal {
 		compileStructLitInKeyVal(ctx, v.Elts, t, typ)
+		if hasPtr {
+			ctx.cb.UnaryOp(gotoken.AND)
+		}
 		return
 	}
-	compileCompositeLitElts(ctx, v.Elts, kind)
+	compileCompositeLitElts(ctx, v.Elts, kind, &kvType{underlying: underlying})
 	n := len(v.Elts)
 	if typ == nil {
+		if kind == compositeLitVal && n > 0 {
+			panic("TODO: mapLit should be in {key: val, ...} form")
+		}
 		ctx.cb.MapLit(nil, n<<1)
 		return
 	}
@@ -379,6 +619,9 @@ func compileCompositeLit(ctx *blockCtx, v *ast.CompositeLit) {
 	default:
 		log.Panicln("compileCompositeLit: unknown type -", reflect.TypeOf(underlying))
 	}
+	if hasPtr {
+		ctx.cb.UnaryOp(gotoken.AND)
+	}
 }
 
 func compileSliceLit(ctx *blockCtx, v *ast.SliceLit) {
@@ -409,9 +652,10 @@ func comprehensionKind(v *ast.ComprehensionExpr) int {
 	panic("TODO: invalid comprehensionExpr")
 }
 
-// {for k, v <- listOrMap, cond}
-// {expr for k, v <- listOrMap, cond}
-// {kexpr: vexpr for k, v <- listOrMap, cond}
+// [expr for k, v <- container, cond]
+// {for k, v <- container, cond}
+// {expr for k, v <- container, cond}
+// {kexpr: vexpr for k, v <- container, cond}
 func compileComprehensionExpr(ctx *blockCtx, v *ast.ComprehensionExpr, twoValue bool) {
 	kind := comprehensionKind(v)
 	pkg, cb := ctx.pkg, ctx.cb
@@ -445,9 +689,12 @@ func compileComprehensionExpr(ctx *blockCtx, v *ast.ComprehensionExpr, twoValue
 		names = append(names, forStmt.Value.Name)
 		cb.ForRange(names...)
 		compileExpr(ctx, forStmt.X)
-		cb.RangeAssignThen()
+		cb.RangeAssignThen(forStmt.TokPos)
 		if forStmt.Cond != nil {
 			cb.If()
+			if forStmt.Init != nil {
+				compileStmt(ctx, forStmt.Init)
+			}
 			compileExpr(ctx, forStmt.Cond)
 			cb.Then()
 			end++
@@ -498,10 +745,13 @@ var (
 
 func compileErrWrapExpr(ctx *blockCtx, v *ast.ErrWrapExpr) {
 	pkg, cb := ctx.pkg, ctx.cb
-	inGlobal := (cb.Scope().Parent() == types.Universe) // not in a function body
+	useClosure := (v.Tok == token.NOT || v.Default != nil)
+	if !useClosure && (cb.Scope().Parent() == types.Universe) {
+		panic("TODO: can't use expr? in global")
+	}
 	ret := pkg.NewAutoParam("_gop_ret")
 	sig := types.NewSignature(nil, nil, types.NewTuple(ret), false)
-	if inGlobal {
+	if useClosure {
 		cb.NewClosureWith(sig).BodyStart(pkg)
 	} else {
 		cb.CallInlineClosureStart(sig, 0, false)
@@ -515,16 +765,13 @@ func compileErrWrapExpr(ctx *blockCtx, v *ast.ErrWrapExpr) {
 	if v.Tok == token.NOT { // expr!
 		cb.Val(pkg.Builtin().Ref("panic")).Val(err).Call(1).EndStmt() // TODO: wrap err
 	} else if v.Default == nil { // expr?
-		if inGlobal {
-			panic("TODO: can't use expr? in global")
-		}
 		cb.Val(err).ReturnErr(true) // TODO: wrap err & return err
 	} else { // expr?:val
 		compileExpr(ctx, v.Default)
 		cb.Return(1)
 	}
 	cb.End().Return(0).End()
-	if inGlobal {
+	if useClosure {
 		cb.Call(0)
 	}
 }
