@@ -24,6 +24,7 @@ import (
 
 	goast "go/ast"
 	gotoken "go/token"
+	"go/types"
 
 	"github.com/goplus/gop/ast"
 	"github.com/goplus/gop/token"
@@ -43,7 +44,10 @@ func commentStmt(ctx *blockCtx, stmt ast.Stmt) {
 	if ctx.fileLine {
 		start := stmt.Pos()
 		pos := ctx.fset.Position(start)
-		line := fmt.Sprintf("\n//line %s:%d", relFile(ctx.targetDir, pos.Filename), pos.Line)
+		if ctx.relativePath {
+			pos.Filename = relFile(ctx.targetDir, pos.Filename)
+		}
+		line := fmt.Sprintf("\n//line %s:%d", pos.Filename, pos.Line)
 		comments := &goast.CommentGroup{
 			List: []*goast.Comment{{Text: line}},
 		}
@@ -58,16 +62,14 @@ func compileStmts(ctx *blockCtx, body []ast.Stmt) {
 }
 
 func compileStmt(ctx *blockCtx, stmt ast.Stmt) {
-	defer func() {
-		if e := recover(); e != nil {
-			if err, ok := e.(error); ok {
-				ctx.handleErr(err)
+	if enableRecover {
+		defer func() {
+			if e := recover(); e != nil {
+				ctx.handleRecover(e)
 				ctx.cb.ResetStmt()
-			} else {
-				panic(e)
 			}
-		}
-	}()
+		}()
+	}
 	commentStmt(ctx, stmt)
 	switch v := stmt.(type) {
 	case *ast.ExprStmt:
@@ -115,8 +117,29 @@ func compileStmt(ctx *blockCtx, stmt ast.Stmt) {
 }
 
 func compileReturnStmt(ctx *blockCtx, expr *ast.ReturnStmt) {
-	for _, ret := range expr.Results {
-		compileExpr(ctx, ret)
+	var n = -1
+	var results *types.Tuple
+	for i, ret := range expr.Results {
+		if c, ok := ret.(*ast.CompositeLit); ok && c.Type == nil {
+			if n < 0 {
+				results = ctx.cb.Func().Type().(*types.Signature).Results()
+				n = results.Len()
+			}
+			var typ types.Type
+			if i < n {
+				typ = results.At(i).Type()
+			}
+			compileCompositeLit(ctx, c, typ, true)
+		} else {
+			twoValue := false
+			if len(expr.Results) == 1 {
+				if _, ok := ret.(*ast.ComprehensionExpr); ok {
+					results = ctx.cb.Func().Type().(*types.Signature).Results()
+					twoValue = (results.Len() == 2)
+				}
+			}
+			compileExpr(ctx, ret, twoValue)
+		}
 	}
 	ctx.cb.Return(len(expr.Results), expr)
 }
@@ -145,12 +168,14 @@ func compileAssignStmt(ctx *blockCtx, expr *ast.AssignStmt) {
 			}
 		}
 		ctx.cb.DefineVarStart(expr.Pos(), names...)
-		defer func() {
-			if e := recover(); e != nil {
-				ctx.cb.ResetInit()
-				panic(e)
-			}
-		}()
+		if enableRecover {
+			defer func() {
+				if e := recover(); e != nil {
+					ctx.cb.ResetInit()
+					panic(e)
+				}
+			}()
+		}
 		for _, rhs := range expr.Rhs {
 			compileExpr(ctx, rhs, twoValue)
 		}
@@ -212,7 +237,11 @@ func compileRangeStmt(ctx *blockCtx, v *ast.RangeStmt) {
 		}
 		compileExpr(ctx, v.X)
 	}
-	cb.RangeAssignThen()
+	pos := v.TokPos
+	if pos == 0 {
+		pos = v.For
+	}
+	cb.RangeAssignThen(pos)
 	compileStmts(ctx, v.Body.List)
 	cb.SetComments(comments, true)
 	cb.End()
@@ -232,7 +261,7 @@ func compileForPhraseStmt(ctx *blockCtx, v *ast.ForPhraseStmt) {
 	}
 	cb.ForRange(names...)
 	compileExpr(ctx, v.X)
-	cb.RangeAssignThen()
+	cb.RangeAssignThen(v.TokPos)
 	if v.Cond != nil {
 		cb.If()
 		compileExpr(ctx, v.Cond)
@@ -477,19 +506,19 @@ func compileDeclStmt(ctx *blockCtx, expr *ast.DeclStmt) {
 		switch d.Tok {
 		case token.TYPE:
 			for _, spec := range d.Specs {
-				loadType(ctx, spec.(*ast.TypeSpec))
+				compileType(ctx, spec.(*ast.TypeSpec))
 			}
 		case token.CONST:
 			for _, spec := range d.Specs {
 				v := spec.(*ast.ValueSpec)
 				names := makeNames(v.Names)
-				loadConsts(ctx, names, v)
+				loadConsts(ctx, names, v) // TODO: check bug
 			}
 		case token.VAR:
 			for _, spec := range d.Specs {
 				v := spec.(*ast.ValueSpec)
 				names := makeNames(v.Names)
-				loadVars(ctx, names, v)
+				loadVars(ctx, names, v) // TODO: check bug
 			}
 		default:
 			log.Panicln("TODO: compileDeclStmt - unknown")
@@ -497,4 +526,13 @@ func compileDeclStmt(ctx *blockCtx, expr *ast.DeclStmt) {
 	}
 }
 
+func compileType(ctx *blockCtx, t *ast.TypeSpec) {
+	name := t.Name.Name
+	if t.Assign != token.NoPos { // alias type
+		ctx.cb.AliasType(name, toType(ctx, t.Type))
+	} else {
+		ctx.cb.NewType(name).InitType(ctx.pkg, toType(ctx, t.Type))
+	}
+}
+
 // -----------------------------------------------------------------------------
