@@ -32,6 +32,7 @@ import (
 	"github.com/goplus/gop/cmd/internal/help"
 	"github.com/goplus/gop/cmd/internal/install"
 	"github.com/goplus/gop/cmd/internal/run"
+	"github.com/goplus/gop/cmd/internal/test"
 )
 
 func mainUsage() {
@@ -48,6 +49,7 @@ func init() {
 		install.Cmd,
 		build.Cmd,
 		clean.Cmd,
+		test.Cmd,
 	}
 }
 
