@@ -64,31 +64,31 @@ func saveGoFile(gofile string, pkg *gox.Package) error {
 	if err != nil {
 		return err
 	}
-	return gox.WriteFile(gofile, pkg)
+	return gox.WriteFile(gofile, pkg, false)
 }
 
-func findGoModFile(dir string) (string, error) {
-	modfile, err := cl.FindGoModFile(dir)
+func findGoModFile(dir string) (modfile string, noCacheFile bool, err error) {
+	modfile, err = cl.FindGoModFile(dir)
 	if err != nil {
 		home := os.Getenv("HOME")
 		modfile = home + "/gop/go.mod"
 		if fi, e := os.Lstat(modfile); e == nil && !fi.IsDir() {
-			return modfile, nil
+			return modfile, true, nil
 		}
 		modfile = home + "/goplus/go.mod"
 		if fi, e := os.Lstat(modfile); e == nil && !fi.IsDir() {
-			return modfile, nil
+			return modfile, true, nil
 		}
 	}
-	return modfile, err
+	return
 }
 
-func findGoModDir(dir string) string {
-	modfile, err := findGoModFile(dir)
+func findGoModDir(dir string) (string, bool) {
+	modfile, nocachefile, err := findGoModFile(dir)
 	if err != nil {
 		log.Fatalln("findGoModFile:", err)
 	}
-	return filepath.Dir(modfile)
+	return filepath.Dir(modfile), nocachefile
 }
 
 func runCmd(cmd *base.Command, args []string) {
@@ -103,17 +103,20 @@ func runCmd(cmd *base.Command, args []string) {
 	} else if *flagDebug {
 		log.SetOutputLevel(log.Ldebug)
 		gox.SetDebug(gox.DbgFlagAll)
+		cl.SetDebug(cl.DbgFlagAll)
 	}
 	if *flagVerbose {
 		gox.SetDebug(gox.DbgFlagAll &^ gox.DbgFlagComments)
+		cl.SetDebug(cl.DbgFlagAll)
+		cl.SetDisableRecover(true)
 	} else if *flagAsm {
 		gox.SetDebug(gox.DbgFlagInstruction)
 	}
 	if *flagProf {
 		panic("TODO: profile not impl")
 	}
-	fset := token.NewFileSet()
 
+	fset := token.NewFileSet()
 	src, _ := filepath.Abs(flag.Arg(0))
 	isDir, err := IsDir(src)
 	if err != nil {
@@ -124,16 +127,19 @@ func runCmd(cmd *base.Command, args []string) {
 		return
 	}
 
-	var targetDir, file, gofile string
+	var srcDir, file, gofile string
 	var pkgs map[string]*ast.Package
 	if isDir {
-		targetDir = src
+		srcDir = src
 		gofile = src + "/gop_autogen.go"
 		pkgs, err = parser.ParseDir(fset, src, nil, 0)
 	} else {
-		targetDir, file = filepath.Split(src)
-		targetDir = filepath.Join(targetDir, ".gop")
-		gofile = filepath.Join(targetDir, file+".go")
+		srcDir, file = filepath.Split(src)
+		if hasMultiFiles(srcDir, ".gop") {
+			gofile = filepath.Join(srcDir, "gop_autogen_"+file+".go")
+		} else {
+			gofile = srcDir + "/gop_autogen.go"
+		}
 		pkgs, err = parser.Parse(fset, src, nil, 0)
 	}
 	if err != nil {
@@ -144,15 +150,16 @@ func runCmd(cmd *base.Command, args []string) {
 	mainPkg, ok := pkgs["main"]
 	if !ok {
 		if len(pkgs) == 0 { // not a Go+ package, try runGoPkg
-			runGoPkg(src, args)
+			runGoPkg(src, args, true)
 			return
 		}
 		fmt.Fprintln(os.Stderr, "TODO: not a main package")
 		os.Exit(12)
 	}
 
+	modDir, noCacheFile := findGoModDir(srcDir)
 	conf := &cl.Config{
-		Dir: findGoModDir(targetDir), TargetDir: targetDir, Fset: fset, CacheLoadPkgs: true}
+		Dir: modDir, TargetDir: srcDir, Fset: fset, CacheLoadPkgs: true, PersistLoadPkgs: !noCacheFile}
 	out, err := cl.NewPackage("", mainPkg, conf)
 	if err != nil {
 		fmt.Fprintln(os.Stderr, err)
@@ -162,15 +169,8 @@ func runCmd(cmd *base.Command, args []string) {
 	if err != nil {
 		log.Fatalln("saveGoFile failed:", err)
 	}
-	err = goRun(gofile, args)
-	if err != nil {
-		switch e := err.(type) {
-		case *exec.ExitError:
-			os.Stderr.Write(e.Stderr)
-		default:
-			log.Fatalln("go run failed:", err)
-		}
-	}
+	conf.PkgsLoader.Save()
+	goRun(gofile, args)
 	if *flagProf {
 		panic("TODO: profile not impl")
 	}
@@ -185,7 +185,7 @@ func IsDir(target string) (bool, error) {
 	return fi.IsDir(), nil
 }
 
-func goRun(target string, args []string) error {
+func goRun(target string, args []string) {
 	dir, file := filepath.Split(target)
 	goArgs := make([]string, len(args)+2)
 	goArgs[0] = "run"
@@ -193,14 +193,24 @@ func goRun(target string, args []string) error {
 	copy(goArgs[2:], args)
 	cmd := exec.Command("go", goArgs...)
 	cmd.Dir = dir
+	cmd.Stdin = os.Stdin
 	cmd.Stdout = os.Stdout
 	cmd.Stderr = os.Stderr
 	cmd.Env = os.Environ()
-	return cmd.Run()
+	err := cmd.Run()
+	if err != nil {
+		switch e := err.(type) {
+		case *exec.ExitError:
+			os.Stderr.Write(e.Stderr)
+			os.Exit(e.ExitCode())
+		default:
+			log.Fatalln("go run failed:", err)
+		}
+	}
 }
 
-func runGoPkg(src string, args []string) {
-	modfile, err := findGoModFile(src)
+func runGoPkg(src string, args []string, doRun bool) {
+	modfile, noCacheFile, err := findGoModFile(src)
 	if err != nil {
 		log.Fatalln("findGoModFile:", err)
 	}
@@ -216,10 +226,11 @@ func runGoPkg(src string, args []string) {
 		loadModes = loadTypes | packages.NeedName | packages.NeedModule
 	)
 	baseConf := &cl.Config{
-		Fset:          token.NewFileSet(),
-		GenGoPkg:      new(gengo.Runner).GenGoPkg,
-		CacheLoadPkgs: true,
-		NoFileLine:    true,
+		Fset:            token.NewFileSet(),
+		GenGoPkg:        new(gengo.Runner).GenGoPkg,
+		CacheLoadPkgs:   true,
+		PersistLoadPkgs: !noCacheFile,
+		NoFileLine:      true,
 	}
 	loadConf := &packages.Config{Mode: loadModes, Fset: baseConf.Fset}
 	pkgs, err := baseConf.Ensure().PkgsLoader.Load(loadConf, pkgPath)
@@ -230,23 +241,48 @@ func runGoPkg(src string, args []string) {
 		fmt.Fprintln(os.Stderr, "TODO: not a main package")
 		os.Exit(12)
 	}
-	goRun(src+"/.", args)
+	baseConf.PkgsLoader.Save()
+	if doRun {
+		goRun(src+"/.", args)
+	}
 }
 
 func runGoFile(src string, args []string) {
 	targetDir, file := filepath.Split(src)
-	targetDir = filepath.Join(targetDir, ".gop", strings.TrimSuffix(file, ".go"))
-	gofile := filepath.Join(targetDir, "gop_autogen.go")
-	b, err := ioutil.ReadFile(src)
-	if err != nil {
-		log.Fatalln("ReadFile failed:", err)
+	if hasMultiFiles(targetDir, ".go") {
+		targetDir = filepath.Join(targetDir, ".gop", strings.TrimSuffix(file, ".go"))
+		gofile := filepath.Join(targetDir, "gop_autogen.go")
+		b, err := ioutil.ReadFile(src)
+		if err != nil {
+			log.Fatalln("ReadFile failed:", err)
+		}
+		os.MkdirAll(targetDir, 0777)
+		err = ioutil.WriteFile(gofile, b, 0666)
+		if err != nil {
+			log.Fatalln("WriteFile failed:", err)
+		}
+		runGoPkg(targetDir, args, false)
+		goRun(src, args)
+	} else {
+		runGoPkg(targetDir, args, true)
 	}
-	os.MkdirAll(targetDir, 0777)
-	err = ioutil.WriteFile(gofile, b, 0666)
-	if err != nil {
-		log.Fatalln("WriteFile failed:", err)
+}
+
+func hasMultiFiles(srcDir string, ext string) bool {
+	var has bool
+	if f, err := os.Open(srcDir); err == nil {
+		defer f.Close()
+		fis, _ := f.ReadDir(-1)
+		for _, fi := range fis {
+			if !fi.IsDir() && filepath.Ext(fi.Name()) == ext {
+				if has {
+					return true
+				}
+				has = true
+			}
+		}
 	}
-	runGoPkg(targetDir, args)
+	return false
 }
 
 // -----------------------------------------------------------------------------
