@@ -126,7 +126,9 @@ func ParseFSDir(fset *token.FileSet, fs FileSystem, path string, filter func(os.
 			continue
 		}
 		fname := d.Name()
-		if strings.HasSuffix(fname, ".gop") && !strings.HasPrefix(fname, "_") && (filter == nil || filter(d)) {
+		ext := filepath.Ext(fname)
+		_, isGop := extGopFiles[ext]
+		if isGop && !strings.HasPrefix(fname, "_") && (filter == nil || filter(d)) {
 			filename := fs.Join(path, fname)
 			if filedata, err := fs.ReadFile(filename); err == nil {
 				if src, err := ParseFSFile(fset, fs, filename, filedata, mode); err == nil {
@@ -151,6 +153,14 @@ func ParseFSDir(fset *token.FileSet, fs FileSystem, path string, filter func(os.
 	return
 }
 
+var (
+	extGopFiles = map[string]int16{
+		".gop": ast.FileTypeGop,
+		".spx": ast.FileTypeSpx,
+		".gmx": ast.FileTypeGmx,
+	}
+)
+
 // -----------------------------------------------------------------------------
 
 // ParseFile parses the source code of a single Go+ source file and returns the corresponding ast.File node.
@@ -176,12 +186,13 @@ func ParseFSFile(fset *token.FileSet, fs FileSystem, filename string, src interf
 // If do this, parsing will display error line number when error occur
 func parseFileEx(fset *token.FileSet, filename string, code []byte, mode Mode) (f *ast.File, err error) {
 	var b bytes.Buffer
-	var isMod, noEntrypoint bool
+	var isMod, noEntrypoint, noPkgDecl bool
 	var fsetTmp = token.NewFileSet()
 	f, err = parseFile(fsetTmp, filename, code, PackageClauseOnly)
 	if err != nil {
 		fmt.Fprintf(&b, "package main;%s", code)
 		code = b.Bytes()
+		noPkgDecl = true
 	} else {
 		isMod = f.Name.Name != "main"
 	}
@@ -206,6 +217,8 @@ func parseFileEx(fset *token.FileSet, filename string, code []byte, mode Mode) (
 		f, err = parseFile(fset, filename, code, mode)
 		if err == nil {
 			f.NoEntrypoint = noEntrypoint
+			f.NoPkgDecl = noPkgDecl
+			f.FileType = extGopFiles[filepath.Ext(filename)]
 		}
 	}
 	return
