@@ -1028,6 +1028,24 @@ func (p *printer) expr1(expr ast.Expr, prec1, depth int) {
 			p.print(token.COLON)
 			p.expr(x.Default)
 		}
+	case *ast.LambdaExpr:
+		if x.LhsHasParen {
+			p.print(token.LPAREN)
+			p.identList(x.Lhs, false)
+			p.print(token.RPAREN, blank)
+		} else if x.Lhs != nil {
+			p.expr(x.Lhs[0])
+			p.print(blank)
+		}
+		p.print(token.RARROW, blank)
+		if x.RhsHasParen {
+			p.print(token.LPAREN)
+			p.exprList(token.NoPos, x.Rhs, 1, noIndent, token.NoPos, false)
+			p.print(token.RPAREN)
+		} else {
+			p.expr(x.Rhs[0])
+		}
+
 	default:
 		log.Fatalf("unreachable %T\n", x)
 	}
@@ -1048,6 +1066,10 @@ func (p *printer) listForPhrase(prev0 token.Pos, list []*ast.ForPhrase, depth in
 		p.expr(x.X)
 		if x.Cond != nil {
 			p.print(x.Cond.Pos(), token.COMMA, blank)
+			if x.Init != nil {
+				p.stmt(x.Init, false)
+				p.print(token.SEMICOLON, blank)
+			}
 			p.expr(x.Cond)
 		}
 	}
@@ -1872,7 +1894,8 @@ func (p *printer) funcDecl(d *ast.FuncDecl) {
 		return
 	}
 
-	p.print(d.Pos(), token.FUNC, blank)
+	pos := d.Pos()
+	p.print(pos, token.FUNC, blank)
 	// We have to save startCol only after emitting FUNC; otherwise it can be on a
 	// different line (all whitespace preceding the FUNC is emitted only when the
 	// FUNC is emitted).
@@ -1932,7 +1955,7 @@ func (p *printer) declList(list []ast.Decl) {
 			// only print line break if we are not at the beginning of the output
 			// (i.e., we are not printing only a partial program)
 			min := 1
-			if prev != tok || getDoc(d) != nil {
+			if tok == token.FUNC || tok == token.TYPE || prev != tok || getDoc(d) != nil {
 				min = 2
 			}
 			// start a new section if the next declaration is a function
@@ -1952,8 +1975,10 @@ func (p *printer) file(src *ast.File) {
 		}
 	}
 	p.setComment(src.Doc)
-	p.print(src.Pos(), token.PACKAGE, blank)
-	p.expr(src.Name)
+	if !src.NoPkgDecl {
+		p.print(src.Pos(), token.PACKAGE, blank)
+		p.expr(src.Name)
+	}
 	p.declList(src.Decls)
 	p.print(newline)
 	p.unnamedFuncName = ""
